function imgc = colormask(img, smask, bwidth, cols)
% colormask - generate a colored segmented image
%
%   imgc = colormask(img, smask, bwidth, cols)
%    
%      bwidth -- border width between segments (default=0) [see mask2border]
%        cols -- custom color table [default below] (N x 3) matrix
%
%   colors used: 0. clear         1.0  1.0  1.0 (background class)
%                1. pink          1.0  0.7  0.7
%                2. light green   0.7  1.0  0.7
%                3. indigo        0.7  0.7  1.0
%                4. yellow        1.0  1.0  0.0
%                5. magenta       1.0  0.0  1.0
%                6. cyan          0.0  1.0  1.0
%                7. red           1.0  0.0  0.0
%                8. green         0.0  1.0  0.0
%                9. blue          0.0  0.0  1.0
%               10. white         1.0  1.0  1.0
%
%        imgc -- the video with colored segments
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

% 2008-09-18: fixed bug when smask is 2d
% 2009-10-20: fixed to work with MATLAB 6

if (nargin<3)
  bwidth = 0;
end
if (nargin<4) || isempty(cols)
  % color video w/ regions
  cols=[1 1 1; ...        % no class
	1 0.7 0.7; ...
	0.7 1 0.7; ...
	0.7 0.7 1; ...
	1 1 0; ...
	1 0 1; ...
	0 1 1; ...
	1 0 0; ...
	0 1 0; ...
	0 0 1; ...
	0.5 1 1; ...
	1 1 1;];
end

sy = size(img,1);
sx = size(img,2);
sz = size(img,3);

% no class = no color
imgc1 = reshape(img, [sy sx 1 sz]);
imgc2 = reshape(img, [sy sx 1 sz]);
imgc3 = reshape(img, [sy sx 1 sz]);

K = double(max(smask(:)));
myclass = class(smask);

is2d = (ndims(smask)==2);

for j=1:K
  jj = feval(myclass, j);   % convert to the same class as smask
  ii = find(smask == jj);
  if (is2d)
    for t=1:sz
      offset = (t-1)*(sx*sy);
      imgc1(ii+offset) = imgc1(ii+offset) * cols(j+1,1);
      imgc2(ii+offset) = imgc2(ii+offset) * cols(j+1,2);
      imgc3(ii+offset) = imgc3(ii+offset) * cols(j+1,3);
    end    
  else
    imgc1(ii) = imgc1(ii) * cols(j+1,1);
    imgc2(ii) = imgc2(ii) * cols(j+1,2);
    imgc3(ii) = imgc3(ii) * cols(j+1,3);
  end
end

% create a white border
if (bwidth > 0)
  ii = mask2border(smask, bwidth);

  %ii = find(bmask);
  if (is2d)
    for t=1:sz
      offset = (t-1)*(sx*sy);
      imgc1(ii+offset) = 255;
      imgc2(ii+offset) = 255;
      imgc3(ii+offset) = 255;
    end
  else
    imgc1(ii) = 255;
    imgc2(ii) = 255;
    imgc3(ii) = 255;
  end
end

imgc = cat(3, imgc1, imgc2, imgc3);

