function f = feature_ellipsoid(img, segm, pmap, roi, opt, debug)
% feature_ellipsoid - ellipsoid features
%
%    f = feature_ellipsoid(img, segm, pmap, roi, opt, debug)
%
%    f(1) = center y
%    f(2) = center x
%    f(3) = eccentricity
%    f(4) = orientation
%    f(5) = major axis length
%    f(6) = minor axis length
%
% Copyright 2008, Antoni Chan, SVCL, UCSD


m = {'centroid','eccentricity','orientation','majoraxislength','minoraxislength'};

rp = regionprops(double(segm.*roi), m);

f = [rp.Centroid(2); rp.Centroid(1); rp.Eccentricity; rp.Orientation; ...
     rp.MajorAxisLength; rp.MinorAxisLength];

if (debug)
  figure
  subplot(1,2,1)
  imagesc(segm)
  subplot(1,2,2)
  imagesc(segm.*roi)
  f'
end

