function f = feature_norm_blobhist(img, blobs, pmap, roi, opt, debug)
% feature_norm_blobhist - normalized blob histogram
%
%    f = feature_norm_blobhist(img, blobs, pmap, roi, opt, debug)
%
%      opt = [numbins, binsize]
%
% Copyright 2008, Antoni Chan, SVCL, UCSD


MINBLOBSIZE = 10;

bins    = opt(1);
binsize = opt(2);

% get number of blobs
maxj    = double(max(blobs(:)));

% mask with ROI
myblobs = double(blobs).*roi;

blobsizes = [];
for j=1:maxj
  % get blob
  jblob = (myblobs == j);
  
  % get area & normalized area
  tmp  = sum(sum(jblob));
  tmp2 = sum(sum(jblob.*pmap.pmapxy));
  
  % if larger than minimum size, then remember
  if (tmp >= MINBLOBSIZE)
    blobsizes(end+1) = tmp2;
  end  
end

% take histogram
hedges      = 0:binsize:(binsize*bins);
hedges(end) = inf;      % last bin catches everything else
if ~isempty(blobsizes)
  tmpf = histc(blobsizes, hedges);
else
  tmpf = zeros(size(hedges));
end

% histogram
f = tmpf(1:bins);
f = f(:);

if (debug)
  figure
  subplot(2,2,1)
  imagesc(myblobs)
  subplot(2,2,2)
  imagesc(pmap.pmapxy);
  colorbar
  subplot(2,2,3)
  plot(blobsizes, 'x');
  hold on
  plot(repmat([0;maxj],[1 bins]), repmat(hedges(1:bins),[2 1]), 'r-');
  hold off
  xlabel('blobs');
  ylabel('norm size');
  subplot(2,2,4)
  bar(hedges(1:bins)+binsize/2, f);
  colormap gray
end

