function m = getminkowski(eimg, pmap, debug)
% getminkowski - compute the minkowski fractal dimension
%
%   m = getminkowski(eimg, pmap, debug)
% 
%    eimg - edge image
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

if all(eimg(:) == 0)
  m = 0;
  return
end

disksize = 1:15;

area = zeros(1, length(disksize));

for d=disksize
  % get disk
  se = strel('disk', d);
  
  % dilate image
  dimg = imdilate(eimg, se);
  
  % get normalized area
  area(d) = sum(sum(dimg.*pmap.pmapxy));
  
  if (debug)
    figure(1000)
    subplot(4,4,d)
    imagesc(dimg);
    title(sprintf('a=%g', area(d)));
    colormap gray
  end
end

% compute log area and disksize
logarea     = log(area);
logdisksize = log(disksize);

% do linear regression
b = regress(logarea(:), [ones(length(disksize),1) logdisksize(:)]);

% fractal dimension = 2-slope
m = 2-b(2);

if (debug)
  figure(1001)
  plot(logdisksize, logarea, 'bx');
  hold on
  plot(logdisksize, b(2)*logdisksize+b(1), 'r-');
  hold off
  grid
  xlabel('log(disksize)');
  ylabel('log(area)');
end

