function angle_hist=getorientation(edgeimage,edgedens,num_bins)
% getorientation - get a histogram of the orientation of the edges of the segmentation
%
%   angle_hist=getorientation(edgeimage,edgedens,num_bins)
%
%  i-th bin is (i-1)*pi/num_bins radians
%  bin 1 is vertical edges, bin num_bins/2 is horizontal edges
%  
% uses oriented Gaussians
%
% Copyright 2008, Antoni Chan, SVCL, UCSD


% make sure a double
z = double(edgeimage);

% get active points
ii = find(edgeimage);

% no edges!
if (isempty(ii))
  angle_hist = zeros(num_bins, 1);
  return
end

%figure

% now filter with oriented gaussians
s1=2; % 3x stddev
s2=2*3*3;
u=-8:8;
[x,y] = meshgrid(u,u);
V = [x(:) y(:)]';
ozii = zeros(length(ii),num_bins);
for i=1:num_bins
  % compute covariance matrix
  theta = (i-1)*pi/num_bins;
  rotM = rotatemat2(theta);
  S    = inv(rotM*diag([s1 s2])*rotM');

  % filter with gaussian
  og = exp(-0.5*sum((S*V).*V,1));
  og = reshape(og, length(u), length(u));
  oz = filter2(og,z);

  % save active points
  ozii(:,i) = oz(ii);
  
  %subplot(2,num_bins,i)
  %imagesc(og);
  %axis equal
  %subplot(2,num_bins,i+num_bins)
  %imagesc(oz);
end

% pick maximum filter response
[foo, od] = max(ozii, [], 2);

% compute histogram
angle_hist = accumarray(od(:), edgedens(ii), [num_bins 1]);


%figure
%subplot(2,2,1)
%imagesc(g);
%axis equal
%subplot(2,2,2)
%imagesc(z);
%subplot(2,2,3)
%tmp = zeros(size(edgeimage));
%tmp(ii) = foo;
%imagesc(tmp);
%colorbar
%subplot(2,2,3);
%tmp(ii) = od;
%imagesc(tmp);
%colorbar
%subplot(2,2,4);
%bar(angle_hist);

function M = rotatemat2(th)
% rotatemat - 2d rotation matrix
%
%   M = rotatemat(th)
%
%   th = radians rotation from x-axis (in coordinate space)

M = [cos(th), sin(th); -sin(th), cos(th)];

