function makedir(f, verbose)
% makedir - make a directory for file
%
%    makedir(f, verbose)
%
% make the directories required to save f.
%  
% Copyright 2008, Antoni Chan, SVCL, UCSD


if (nargin<2)
  verbose = 0;
end

ds = split(f, filesep);

if (f(1) == filesep)
  mydir = '';
else
  mydir = '.';
end

if (verbose)
  fprintf('target: %s\n', f);
end

for i=1:length(ds)-1
  newdir = [mydir filesep ds{i}];
  
  if (verbose)
    fprintf('checking: %s ', newdir);
  end
  
  if ~exist(newdir, 'dir')
    if (verbose)
      fprintf('[create]\n');
    end
    [s, msg] = mkdir(mydir, ds{i});
    if (s == 0)
      error('mkdir error!');
    end
    if (s == 1) && ~isempty(msg)
      msg
      error('mkdir warning!');
    end
  else
    if (verbose)
      fprintf('[exists]\n');
    end
  end
  
  mydir = newdir;
end

function out = split(in, tok)
% split - split a string with a token
%
%    out = split(instr, tok)
%
%  default tok=<tab>

if (nargin<2)
  tok = sprintf('\t');
end

i=1;
while (length(in) > 0)
  [t, in] = strtok(in, tok);
  out{i} = t;
  i = i+1;
end
