function [fvall, gtall] = peoplecnt_extractdata(opt, dirsind, f_all, cgt)
% peoplecnt_extractdata - extract relevant data
%
%  [fvall, gtall] = peoplecnt_extractdata(opt, dirsind, f_all, cgt)
%
%       opt - options structure
%   dirsind - the index of the direction to extact
%     f_all - the feature vectors (from peoplecnt_features)
%       cgt - the counting ground-truth (from peoplecnt_loadgt)
%     fvall - the feature vectors for this direction
%     gtall - the ground-truth counts for this direction
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

%%% extract the relevant data %%%
gtinds = [];
fvall  = [];
gtall  = [];

for i=1:length(cgt)
  if (~isempty(cgt{i}))
    gtinds(end+1) = i;
    
    % for this direction...
    % ...append the feature vectors
    if (~isempty(f_all))
      fvall = [fvall, f_all{i}{dirsind}];
    end
      
    % find the GT direction
    myind = find(strcmp(opt.dirs{dirsind}, cgt{i}.dir));
    if (isempty(myind))
      opt.dirs{dirsind}
      cgt{i}.dirs
      error('could not find direction in GT!');
    end
      
    % ...append count
    gtall = [gtall, cgt{i}.count{myind}];      
  end  
end
	
fprintf('- GT indexes: ');
fprintf('%d ', gtinds);
fprintf('\n');

if ~isempty(fvall) && (size(fvall,2) ~= length(gtall))
  error('feature vector and groundtruth size mismatch');
end
      
