function [Ypred, Spred] = peoplecnt_gp_predict(X, gp, opt, f_all, xmean, xstd)
%PEOPLECNT_GP_PREDICT Summary of this function goes here
%   Detailed explanation goes here

%gp.X = gp.X{1};
%gp.Y = gp.Y{1};



% Get ground truth
cgt = peoplecnt_loadgt(opt);

%Get data
[fvall, gtall] = peoplecnt_extractdata(opt, 1, f_all, cgt);

% Set new training data
fvtrain = [];
gttrain = [];
for m = 2:2:length(opt.train2)
  fvtrain = [fvtrain, fvall(:,opt.train2(m-1):opt.train2(m))];
  %fvtrain = repmat(fvtrain, 1, 3);
  gttrain = [gttrain; gtall(opt.train2(m-1):opt.train2(m))'-gp.Ymean];
  %gttrain = repmat(gttrain, 3, 1);
end

%%% normalize %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if any(opt.model.norm == 'X')
  fprintf('  - normalizing (0,1)\n');
  
  % check for xstd=0
  ii = find(xstd == 0);
  if ~isempty(ii)
    fprintf('*** WARNING: these features are empty: ');
    fprintf('%d ', ii);
    fprintf('\n');
    % force no scaling
    xstd(ii) = 1;
  end
elseif any(opt.model.norm == 'x')
  fprintf('  - normalizing mean\n');
  xmean = mean(fvtrain,2);
  xstd  = [];
else
  fprintf('  - no normalization\n');
  xstd  = [];
  xmean = [];
end

% normalize mean
if ~isempty(xmean)
  fvtrain = fvtrain - repmat(xmean, 1, size(fvtrain,2));
end

% normalize std
if ~isempty(xstd)
  fvtrain = fvtrain ./ repmat(xstd, 1, size(fvtrain,2));
end

% get parameters
theta1 = exp(-gp.loghyper(1));
theta2 = exp(gp.loghyper(2));
theta3 = exp(gp.loghyper(3));
theta4 = exp(gp.loghyper(4));

% get predictions
K = peoplecnt_kernel(theta1, theta2, theta3, gp.X, gp.X);
A = theta4.^2 *eye(size(gp.X,2));
L = chol(K+A)';                        % cholesky factorization of the covariance

kstar = peoplecnt_kernel(theta1, theta2, theta3, gp.X, X);
kss = peoplecnt_kernel(theta1, theta2, theta3, X, X);

K_p_p = peoplecnt_kernel(theta1, theta2, theta3, fvtrain, fvtrain);
K_o_p = peoplecnt_kernel(theta1, theta2, theta3, gp.X, fvtrain);
K_p = K_p_p - K_o_p'/L' * (L\K_o_p);
kstar_p = peoplecnt_kernel(theta1, theta2, theta3, fvtrain, X) - ...
  K_o_p'/L' * (L\peoplecnt_kernel(theta1, theta2, theta3, gp.X, X));
L_p = chol(K_p+theta4.^2 *eye(size(fvtrain,2)))';
v = L\kstar;

% Ypred = kstar'/L' * (L\gp.Y');
% Spred = diag(kss) - sum(v.*v)';

mu = kstar'/(K+A)*gp.Y';
sigma = diag(kss) - sum(v.*v)';
Y_p = gttrain - K_o_p'/(K+A)*gp.Y';
v_p = L_p\kstar_p;

Ypred = mu + kstar_p'/L_p'*(L_p\Y_p);
Spred = sigma - sum(v_p.*v_p)';


%[Ypred Spred] = gpr(gp.loghyper, gp.covfunc, gp.X', gp.Y(:), X');

% undo normalization
if any(gp.normmode == 'y')
  Ypred = Ypred + gp.Ymean;   % add mean to Y
end

end

