function blobs = peoplecnt_segmblobs(segm, numcl, blobsname, recompute, flag_nogz)
% peoplecnt_segmblobs - identify individual blobs
%
%   blobs = peoplecnt_segmblobs(segm, numcl, blobsname, recompute, flag_nogz)
%
%           segm - segmentation
%      blobsname - blob file to save to (.mat extension)
%      recompute - force recompute
%      flag_nogz - don't gzip mat file
%       blobs{j} - blobs for segment j
%                  (the last cell is blobs for all classes)
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

% 2008-09-01 - fixed bug with numcl = max(segm), might not match true number of classes

fprintf('*** segmblobs: %s ', blobsname);

if ~recompute && ~isempty(blobsname) && ...
      ((~flag_nogz && exist([blobsname '.gz'], 'file')) || ...
       (flag_nogz && exist(blobsname, 'file')))
  fprintf('[cached]\n');
  if (flag_nogz)
    tmp = load(blobsname);
  else
    tmp = loadgz(blobsname);
  end
  blobs = tmp.blobs;
  
  if (length(blobs) ~= numcl)
    error('CACHE DOES NOT MATCH CLASSES!');
  end
  
else
  fprintf('[blobifying K=%d]', numcl);

  blobs = cell(1,numcl);
  
  %%% for each class %%%
  for j=1:numcl
    jblob = uint8(zeros(size(segm))); 
    
    %%% for each frame %%%
    for i=1:size(segm, 3)
      % identify blobs
      myblob = bwlabel((segm(:,:,i) == j), 4);
      
      % sanity check
      if (any(myblob(:) < 0) || any(myblob(:) > 255))
	error('too many blobs!');
      end
      
      jblob(:,:,i) = uint8(myblob);
    end
    
    blobs{j} = jblob;
  end
  
  %%% total blob %%%
  totblobs = uint8(zeros(size(segm)));
  for i=1:size(segm,3)
    jblob = zeros(size(segm,1), size(segm,2));
    for j=1:numcl
      offset = max(jblob(:));
      myblob = double(blobs{j}(:,:,i));
      jblob  = jblob + (myblob>0).*(myblob+offset);
    end
    
    % sanity check
    if (any(jblob(:) < 0) || any(jblob(:) > 255))
      error('too many blobs!');
    end
    totblobs(:,:,i) = jblob;
  end
  
  blobs{numcl+1} = totblobs;
  
  if (~isempty(blobsname))  
    fprintf('[saving]');	
    makedir(blobsname);
    if (flag_nogz)
      save(blobsname, 'blobs');
    else
      savegz(blobsname, 'blobs');
    end
  end
  fprintf('\n');
end

