function [Ypred, Spred] = gp_predict(X, gp)
% gp_predict - predict with Gaussian Process regression
%
%   [Ypred, Spred] = gp_predict(X, gp)
%
%       X = feature vectors (each column is a feature vector)
%      gp = Gaussian process learned with gp_train
%
%   Ypred = Y predictions
%   Spred = the variance of the predictions
%
% This uses the GPML library by C. E. Rasmussen.
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

% normalize input
if any(gp.normmode == 'x') || any(gp.normmode == 'X')
  X = X - repmat(gp.Xmean,1,size(X,2));
  if any(gp.normmode == 'X')
    X = X./repmat(gp.Xstd,1,size(X,2));
  end  
end

% get predictions
[Ypred Spred] = gpr(gp.loghyper, gp.covfunc, gp.X', gp.Y(:), X');

% undo normalization
if any(gp.normmode == 'y')
  Ypred = Ypred + gp.Ymean;   % add mean to Y
end

