function [A, B] = covPoly2iso(logtheta, x, z);

% Polynomial kernel (2nd order). The covariance function is parameterized as:
%
% k(x^p,x^q) = (x^p'*inv(P)*x^q + 1./t2)^2
%
% where the P matrix is t2 times the unit matrix. The second term plays the
% role of the bias. The hyperparameter is:
%
% logtheta = [ log(sqrt(t2))]
%
% For more help on design of covariance functions, try "help covFunctions".
%
% ABC - 2008-08-25

if nargin == 0, A = '1'; return; end              % report number of parameters

it2 = exp(-2*logtheta);                                         % t2 inverse
d   = 2;

if nargin == 2                                             % compute covariance
  A = (it2*(1+x*x')).^d;
elseif nargout == 2                              % compute test set covariances
  A = (it2*(1+sum(z.*z,2))).^d;
  B = (it2*(1+x*z')).^d;
else                                                % compute derivative matrix
  A = -2*d*(it2*(1+x*x')).^d;

  % derivative wrt d
  %tmp = (it2*(1+x*x')).^d;
  %A = (tmp).*(log(tmp));
end
