% lin_demo - demo of linear and robust linear regression
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

clear
close all

% reset the random number generators
myseed = 981;
rand('state',  myseed);
randn('state', myseed);

% make linear data
ddim = 1;
npts = 20;
nouts = 5;
trueb = 1*randn(ddim,1);
trueo = 10*randn(1,1);
X = 10*randn(ddim,npts);
Y = trueb'*X + trueo;
Xall = 1.3*min(X):0.1:1.3*max(X);
  
% add noise
Y = Y + randn(1,npts);

% add outlier noise
ii = randperm(npts);
outs = ii(1:nouts);
Y(outs) = Y(outs) + 30*(rand(1,nouts));

figure
for j=1:2
  if (j==1)
    % do linear regression
    lm{1}      = lin_train(X, Y);
    [Yhat{1}]  = lin_predict(X, lm{1});
    [Yall{1}]  = lin_predict(Xall, lm{1});
    mytitle = 'linear';
  else
    % do robust linear regression
    lm{2}      = rob_train(X, Y);
    [Yhat{2}]  = rob_predict(X, lm{2});
    [Yall{2}]  = rob_predict(Xall, lm{2});
    mytitle = 'robust linear';
  end
  
  % plot results
  subplot(1,2,j)
  plot(X, Y, 'bx');
  hold on
  plot(X(outs), Y(outs), 'bo');
  plot(Xall, Yall{j}, 'r-');
  hold off
  grid
  title(mytitle);
  legend('data', 'outliers', 'model');  
end
