% svrl_demo - demo on support vector regression
%
% Copyright 2008, Antoni Chan, SVCL, UCSD
 
clear
close all

% reset random number generators
randn('state',1)
rand('state', 1);

% Generate the sinc data
N       = 100;
noise   = 0.1;
X       = 10*[-1:2/(N-1):1];
y       = sin(abs(X))./abs(X);
t       = y + noise*randn(1,N);

fprintf('*** SVR with fixed parameters ***\n');

% train SVR (C=1, eps=0.1, kernel=RBF, width=0.1)
mysvr = svrl_train(X, t, 1, 0.1, 2, 0.1);

% predict with SVR
ypred = svrl_predict(X, mysvr);

% get SV
[sv,aly,b] = svrl_getsv(mysvr);
for j=1:length(sv)
  [tmp tmpi] = min(abs(sv(j) - X));
  svy(j) = t(tmpi);
end

% Plot the data and function
figure
subplot(1,2,1)
h_y = plot(X,y,'r--','LineWidth',3);
hold on
h_X = plot(X,t,'k.','MarkerSize',16)
h_p = plot(X, ypred, 'k', 'LineWidth', 3);
h_sv = plot(sv, svy, 'ko', 'LineWidth', 2, 'MarkerSize', 10);
hold off
axis([-10.1 10.1 1.1*[min(t) max(t)]]);
grid
legend([h_X, h_y, h_p, h_sv], 'data points', 'sinc function', 'SVR prediction', 'support vectors');
title('SVR with fixed C=1 eps=0.1 width=0.1');

%%% Train with cross-validation %%%%

fprintf('*** SVR using cross-validation ***\n');

% setup parameter lists
Cs     = [0.5 1 24 ];
es     = [0.05 0.1 0.2];
widths = num2cell([0.05 0.1 0.2 0.5]);

% train SVR with cross-validation (4 splits, 0.80 used for training)
mysvrcv = svrl_traincv(X, t, Cs, es, 2, widths, 4, 0.8);

% predict with SVR
ypredcv = svrl_predict(X, mysvrcv);

% get SV
[svcv,alycv,bcv] = svrl_getsv(mysvrcv);
for j=1:length(svcv)
  [tmp tmpi] = min(abs(svcv(j) - X));
  svycv(j) = t(tmpi);
end

% plot results
subplot(1,2,2)
h_ycv = plot(X,y,'r--','LineWidth',3);
hold on
h_Xcv  = plot(X,t,'k.','MarkerSize',16)
h_pcv  = plot(X, ypredcv, 'k', 'LineWidth', 3);
h_svcv = plot(svcv, svycv, 'ko', 'LineWidth', 2, 'MarkerSize', 10);
hold off
axis([-10.1 10.1 1.1*[min(t) max(t)]]);
grid
legend([h_Xcv, h_ycv, h_pcv, h_svcv], 'data points', 'sinc function', ...
       'SVR prediction', 'support vectors');
title(sprintf('SVR using cross-validation (C=%g, eps=%g, width=%g)', ...
	      mysvrcv.options.C, mysvrcv.options.TubeWidth, mysvrcv.options.KernelParam));
