function [sv, aly, b] = svrl_getsv(mysvr)
% svrl_getsv - get support vectors from SVR model
%
%   [sv, aly, b] = svrl_getsv(svr)
%
%   INPUT
%     svr = SVR model
%
%  OUTPUT
%      sv = support vectors (each column is an SV)
%     aly = alpha*y weight
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

% get the text
md = mysvr.model;

% find line-feeds and carriage-returns
lf = find((md == char(10)) + (md == char(13)));

sv  = [];
aly = [];
cursv = 1;

% for each line
lasti = 1;
for i=1:length(lf)
  % get the line
  ll = md(lasti:lf(i)-1);

  % find spaces
  lls = split(ll, ' ');
  
  switch(i)
   case 1
    % header    
   case 2
    % kernel type    
   case 3
    % kernel parameter -d    
   case 4
    % kernel parameter -g    
   case 5
    % kernel parameter -s    
   case 6
    % kernel parameter -r    
   case 7
    % kernel parameter -u    
   case 8
    % dimension
    d = str2double(lls{1});
   case 9 
    % training size
    N = str2double(lls{1});  
   case 10
    % number of support vectors + 1
    nsv = str2double(lls{1})-1;
   case 11
    % threshold b
    b = str2double(lls{1});

   otherwise
    % support vector!
    
    % allocate
    if isempty(sv)
      cursv = 1;
      sv    = zeros(d, nsv);
      aly   = zeros(1, nsv);
    end
    
    if (cursv > nsv)
      error('too many SV!');
    end
    
    % parse
    aly(cursv) = str2double(lls{1});
    
    % go through each dimension
    for j=2:length(lls)
      if (lls{j}(1) ~= '#')
	tmpl = split(lls{j}, ':');
	tmpi = str2double(tmpl{1});  % feature index
	tmps = str2double(tmpl{2});  % feature value
	if (tmpi > d)
	  error('SV does not match dimension!');
	end
	sv(tmpi, cursv) = tmps;
      end
    end
    
    % advance to next
    cursv = cursv+1;
  end  
  
  
  % move to next line
  lasti = lf(i)+1;
end

return

function out = split(in, tok)
% split - split a string with a token
%
%    out = split(instr, tok)
%
%  default tok=<tab>

if (nargin<2)
  tok = sprintf('\t');
end

i=1;
while (length(in) > 0)
  [t, in] = strtok(in, tok);
  out{i} = t;
  i = i+1;
end
