function mysvr = svrl_train(X, Y, C, e, kernel, width)
%   svrl_train - train an SVR regression function
%
%     svr = svrl_train(X, Y, C, e, kernel, width)
%
%    INPUT
%        X = feature vectors (each column is a feature vector)
%        Y = function value
%        C = trade-off parameter
%        e = epsilon insensitivity
%   kernel = the kernel function (default = 2 (rbf))
%    width = the kernel parameters (e.g. width)
%
%   OUTPUT
%      svr = SV regression model
%
%   KERNELS
%     kernel                                           width (parameters)
%     0: linear                                      :
%     1: polynomial (s a*b+c)^d                      : [d s c]
%     2: radial basis function exp(-gamma ||a-b||^2) : [gamma]
%     3: sigmoid tanh(s a*b + c)                     : [s c]
%     4: user defined kernel from kernel.h           : [kernel]
%
% see "svml" for more info on kernel functions.
%
% This function uses the SVMlight library.
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

% find the directory with svm_classify
tmp = which('svm_classify');
tmpk = strfind(tmp, filesep);
basedir = tmp(1:tmpk(end));

oo = svmlopt('Regression', 1, 'TubeWidth', e, 'C', C, 'Kernel', kernel, ...
	     'KernelParam', width, 'ExecPath', basedir, 'Verbosity', 0);

fname = sprintf('/tmp/svml_%d.model', round(rand*1000000));

mysvr = svml(fname, oo);
mysvr = svmltrain(mysvr, X', Y(:));

% read in file
fid         = fopen(fname, 'r');
F           = fread(fid);
fclose(fid);

mysvr.model = char(F');
mysvr.fname = '';

% clean up
delete(fname);
