%
% Example on how to compute a perspective map.
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

clear
close all

% load the two reference videos
tmp =  load('example/Data/vidf1_33_000.mat');
img  = tmp.img;
tmp  = load('example/Data/vidf1_33_001.mat');
img2 = tmp.img;

% frame for the two reference image
N1 = 1;
N2 = 40;
  
% [y,x] coordinates of the ground-plane:
%
%         d--------c
%        /        /
%       /        /
%      /        /
%      a--------b
%
a = [131, 6];    
b = [131, 156];
c = [58 233];
d = [58 135];

% the reference height of a person on line "ab"
refy_close = 30;
% the reference height of the same person on line "cd"
refy_far   = 19;


% make a plot showing everything
polyy = [a(1) b(1) c(1) d(1) a(1)];
polyx = [a(2) b(2) c(2) d(2) a(2)];

figure
subplot(1,2,1)
imagesc(img(:,:,N1));
hold on
colormap gray
plot(polyx, polyy, 'r');
hold off
axis image
subplot(1,2,2)
imagesc(img2(:,:,N2));
hold on
colormap gray
plot(polyx, polyy, 'r');
hold off
axis image

% x-distance between a and b
refx = b(2)-a(2);

% compute the perspective map
pmapx = zeros(size(img,1), size(img,2));
pmapy = zeros(size(img,1), size(img,2));

% for each line...
for y=1:size(img,1)
  % compute the distance between "ad" and "bc" at this y
  fx = ((y-b(1)) / (c(1)-b(1))) * (c(2)-b(2)) + b(2);
  ex = ((y-a(1)) / (d(1)-a(1))) * (d(2)-a(2)) + a(2);
  x = fx-ex;
  
  % inverse of the relative width of x to the "ab"
  pmapx(y,:) = refx/x;
  
  % interpolate the reference height in y
  py = (refy_close-refy_far)/(a(1)-c(1))*(y-c(1)) + refy_far;
  
  % inverse of the relative height of py to the reference height
  pmapy(y,:) = refy_close/py;
end

% multiply pmapy and pmapx to get the perspective map
pmap2 = pmapx.*pmapy;
dmap.pmapxy = pmap2;
dmap.pmapx  = pmapx;
dmap.pmapy  = pmapy;

figure
imagesc(pmap2);
colorbar
axis image
hold on
[cc, ch] = contour(dmap.pmapxy, [1 2 3 4 5], 'r:');
set(ch, 'linewidth', 1);
set(ch, 'linestyle', ':');
cc2 = unique(cc(2,:));
myfont = 18;
for i=1:length(cc2)
  if (cc2(i) < size(img,1))
    text(242, cc2(i), sprintf('%d', round(pmap2(round(cc2(i)),1))), 'color', [0 0 0], 'fontsize', myfont-4)
  end
end
hold off

